@@ -73,7 +73,7 @@ lsm.basis.default = function(object, trms, xlev, grid, ...) {
 # Later addition: na.action arg req'd - vector of row indexes removed due to NAs
 recover.data.call = function(object, trms, na.action, data = NULL, params = NULL, ...) {
     fcall = object # because I'm easily confused
-    vars = setdiff(All.vars(trms), params)
+    vars = setdiff(.all.vars(trms), params)
     if (length(vars) == 0)
         return("Model must have at least one predictor")
     tbl = data
@@ -104,7 +104,7 @@ recover.data.call = function(object, trms, na.action, data = NULL, params = NULL
     
     attr(tbl, "call") = object # the original call
     attr(tbl, "terms") = trms
-    attr(tbl, "predictors") = setdiff(All.vars(delete.response(trms)), params)
+    attr(tbl, "predictors") = setdiff(.all.vars(delete.response(trms)), params)
     attr(tbl, "responses") = setdiff(vars, union(attr(tbl, "predictors"), params))
     tbl
 }
@@ -181,9 +181,9 @@ lsm.basis.merMod = function(object, trms, xlev, grid, vcov., ...) {
     dfargs = misc = list()
     if (lme4::isLMM(object)) {
         pbdis = .lsm.is.true("disable.pbkrtest")
-        sizelim = get.lsm.option("pbkrtest.limit")
-        objsize = object.size(object) / 2^30
-        toobig = objsize > sizelim
+        Nlim = get.lsm.option("pbkrtest.limit")
+        objN = lme4::getME(object, "N")
+        toobig = objN > Nlim
         if (!pbdis && !toobig && requireNamespace("pbkrtest") && missing(vcov.)) {
             dfargs = list(unadjV = V, 
                 adjV = pbkrtest::vcovAdj.lmerMod(object, 0))
@@ -200,9 +200,10 @@ lsm.basis.merMod = function(object, trms, xlev, grid, vcov., ...) {
             if(!pbdis && !("pbkrtest" %in% row.names(installed.packages())))
                 message("Install package 'pbkrtest' to obtain bias corrections and degrees of freedom")
             else if(toobig)
-                message("Adjusted covariance calculations and K-R degrees of freedom\n",
-                        "have been disabled because the model object exceeds ", signif(sizelim, 2), " GB.\n",
-                        "To enable it, set lsm.options(pbkrtest.limit = ", round(objsize+.01, 2), ") or larger,\n",
+                message("Note: Adjusted covariance and degrees-of-freedom calculations have been\n",
+                        "disabled because the number of observations exceeds ", Nlim, ".\n",
+                        "Standard errors and tests may be more biased than if they were adjusted.\n",
+                        "To enable adjustments, set lsm.options(pbkrtest.limit = ", objN, ") or larger,\n",
                         "but be warned that this may result in large computation time and memory use.")
             dffun = function(k, dfargs) NA
         }
@@ -294,13 +295,13 @@ recover.data.lme = function(object, data, ...) {
     recover.data(fcall, delete.response(terms(object)), object$na.action, data = data, ...)
 }
 
-lsm.basis.lme = function(object, trms, xlev, grid, adjustSigma = TRUE, ...) {
+lsm.basis.lme = function(object, trms, xlev, grid, sigmaAdjust = TRUE, ...) {
     contrasts = object$contrasts
     m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
     X = model.matrix(trms, m, contrasts.arg = contrasts)
     bhat = nlme::fixef(object)
     V = .my.vcov(object, ...)
-    if (adjustSigma && object$method == "ML") 
+    if (sigmaAdjust && object$method == "ML") 
         V = V * object$dims$N / (object$dims$N - nrow(V))
     misc = list()
     if (!is.null(object$family)) {
@@ -657,8 +658,8 @@ lsm.basis.gam = function(object, trms, xlev, grid, ...) {
 #     recover.data(fcall, delete.response(terms(object$lme)), object$lme$na.action, ...)
 # }
 # 
-# lsm.basis.gamm = function (object, trms, xlev, grid, adjustSigma = TRUE, ...) {
-#     lsm.basis(object$lme, trms, xlev, grid, adjustSigma, ...)
+# lsm.basis.gamm = function (object, trms, xlev, grid, sigmaAdjust = TRUE, ...) {
+#     lsm.basis(object$lme, trms, xlev, grid, sigmaAdjust, ...)
 #     # Doesn't work because needs the matrices in object$lme$data
 # }
 
@@ -694,7 +695,7 @@ lsm.basis.gam = function(object, trms, xlev, grid, ...) {
 
 ## Alternative to all.vars, but keeps vars like foo$x and foo[[1]] as-is
 ##   Passes ... to all.vars
-All.vars = function(expr, retain = c("\\$", "\\[\\[", "\\]\\]"), ...) {
+.all.vars = function(expr, retain = c("\\$", "\\[\\[", "\\]\\]"), ...) {
     if (!inherits(expr, "formula")) {
         expr = try(eval(expr), silent = TRUE)
         if(inherits(expr, "try-error")) {
@@ -711,3 +712,18 @@ All.vars = function(expr, retain = c("\\$", "\\[\\[", "\\]\\]"), ...) {
         vars = gsub(repl[i], retain[i], vars)
     vars
 }
+
+
+### Not-so-damn-smart replacement of diag() that will 
+### not be so quick to assume I want an identity matrix
+### returns matrix(x) when x is a scalar
+.diag = function(x, nrow, ncol) {
+    if(is.matrix(x))
+        diag(x)
+    else if((length(x) == 1) && missing(nrow) && missing(ncol)) 
+        matrix(x)
+    else 
+        diag(x, nrow, ncol)
+}
+
+
